import os
import re
import subprocess
import sys


def autodetect():
    """Returns a tuple (user, repo). Tries to parse git remote url and also
    Travis CI environment variable."""
    slug = os.environ.get('TRAVIS_REPO_SLUG')
    if slug:
        user, repo = slug.split('/')
        return user, repo

    try:
        remote = subprocess.check_output("git remote get-url origin".split()).decode()
        match = re.match(r"git@github.com:(.+)\/(.+)\.git", remote)
        if match:
            user, repo = match.groups()
            return user, repo
    except subprocess.CalledProcessError as ex:
        print(ex, file=sys.stderr)

    # autodetect failed
    return None, None



import json
from http import client
from urllib.parse import urlparse


URL_FORMAT = "{server}/badge/{user}/{repo}/{subject}"
SVG_URL_FORMAT = "{server}/badge/{user}/{repo}/{subject}.svg"


def _put(url, headers, body, redirects=5):
    parts = urlparse(url)
    ctr = client.HTTPSConnection if parts.scheme == 'https' else client.HTTPConnection
    connection = ctr(host=parts.hostname, port=parts.port)
    connection.request(
        'PUT', parts.path,
        body=body, headers=headers)
    response = connection.getresponse()
    if response.status in (301, 302, 307, 308):
        if redirects == 0:
            raise client.HTTPException("Too many redirects")
        _put(response.headers.get('location'), headers, body, redirects=redirects - 1)
    return response


def upload(server: str, user: str, repo: str, subject: str, status: str, color: str) -> None:
    url = URL_FORMAT.format(server=server, user=user, repo=repo, subject=subject)
    r = _put(url,
             headers={"Content-Type": "application/json"},
             body=json.dumps(dict(status=status, color=color)))
    if r.status >= 200 and r.status < 300:
        return SVG_URL_FORMAT.format(
            server=server, user=user, repo=repo, subject=subject)

    raise client.HTTPException("HTTP Error: {}".format(r.status))


def _mix_colors(c1, c2, v):
    """Mixes two colors according to v in the 0-1 range
    where 0 only r1 and 1 only r2 and e.g. 0.5 is exactly in between"""
    r1, g1, b1 = c1
    r2, g2, b2 = c2
    return (int(r2 * v + r1 * (1 - v)),
            int(g2 * v + g1 * (1 - v)),
            int(b2 * v + b1 * (1 - v)))


def _format(color):
    """Formats an r, g, b tuple as a hex encoded string"""
    return '{:02x}{:02x}{:02x}'.format(*color)


def get_color(min_value: int, max_value: int, value: int) -> str:
    return to_color((value - min_value) / (max_value - min_value))


def to_color(v: float) -> str:
    """Converts a coverage percentage to a color"""
    gradient_stops = [
        (0.5, (0xe0, 0x5d, 0x44)),
        (0.9, (0x44, 0xcc, 0x11)),
    ]
    if v < gradient_stops[0][0]:
        return _format(gradient_stops[0][1])
    if v > gradient_stops[-1][0]:
        return _format(gradient_stops[1][1])

    a = (v - gradient_stops[0][0]) / (gradient_stops[1][0] - gradient_stops[0][0])
    return _format(_mix_colors(gradient_stops[0][1], gradient_stops[1][1], a))



import argparse
import sys
import re

import badges_io.slug as slug
from badges_io.badges import upload, get_color


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--url", help="Where to post badge data", default='https://badges.vidstige.se')
    parser.add_argument("--user", help="User part of slug. Defaults to autodetect")
    parser.add_argument("--repo", help="Repo part of slug. Defaults to autodetect")
    parser.add_argument("--min", help="Max value for auto-computed color", type=int, default=0)
    parser.add_argument("--max", help="Max value for auto-computed color", type=int, default=100)
    parser.add_argument("--value", help="Value to compute color from", type=int, default=None)
    parser.add_argument('subject', help="Subject, e.g. 'coverage'")
    parser.add_argument('status', help="Subject, e.g. '97%'")
    parser.add_argument('color', help="Color, e.g. 'green' or a hex string", nargs='?')
    args = parser.parse_args()

    auto_user, auto_repo = slug.autodetect()
    user, repo = args.user or auto_user, args.repo or auto_repo
    if user and repo:
        color = args.color
        if color is None:
            value = args.value
            if value is None:
                match = re.search(r'[\d\.]+', args.status)
                if match:
                    value = float(match.group())
            if value is None:
                print('No color nor value specified', file=sys.stderr)
            color = get_color(args.min, args.max, value)

        svg = upload(
            args.url, user, repo,
            args.subject, args.status, color)
        print(svg)
    else:
        print("Could not auto detect slug and --user and --repo not given", file=sys.stderr)
        return -1

    return 0


if __name__ == "__main__":
    sys.exit(main())



import os
from setuptools import setup

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

def read(fname):
    with open(fname) as f:
        return f.read()

setup(
    name='badges_io',
    packages=['badges_io'],
    version='1.0.6',
    description='Uploads total coverage for displaying badge',
    long_description=read('README.md'),
    long_description_content_type='text/markdown',
    install_requires=[],
    author='Samuel Carlsson',
    author_email='samuel.carlsson@volumental.com',
    url='https://github.com/Volumental/badges',
    keywords=['badge', 'shield', 'shields.io'],
)
